@@ -1,5 +1,5 @@
 #
-# $Id: Perl6.pm 24448 2008-12-17 20:03:38Z azawawi $
+# $Id: Perl6.pm 24800 2009-01-07 23:12:41Z azawawi $
 #
 package Syntax::Highlight::Perl6;
 
@@ -24,7 +24,7 @@ use STD;
 
 # exports and version
 our @EXPORT_OK = qw();
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 
 # filename constants
 Readonly my $FILE_CSS    => 'p6_style.css';
@@ -59,8 +59,9 @@ sub new {
     $options{utf8_decode} = $options{utf8_decode} // 1;
 
     #is 'text' undefined?
-    croak "'text' option is not found in $class->new"
-        if (!$options{text});
+    if(! defined $options{text}) {
+        croak "'text' option is not found in $class->new"
+    }
 
     my $self = bless \%options, $class;
     $self->{parser} = 0;
@@ -87,6 +88,7 @@ sub _lazy_parse {
         my $len = length $src_text;
         if($len == 0) {
             $src_text = q{ };
+            $len = 1;
         }
         $loc[$len - 1] = [];
 
@@ -297,22 +299,26 @@ sub ansi_text {
 #---------------------------------------------------------------
 # Returns a Perl 5 array containing parse tree records.
 # The array consists of one or more of the following record:
-#   ($position, $buffer, $rule_name, $parse_tree)
+#   ($last_pos, $buffer, $rule, $tree)
 #---------------------------------------------------------------
-sub parse_trees {
+sub tokens {
     my $self = shift;
 
     $self->_lazy_parse();
 
     my %colors = _read_ansi_file();
-    my $parse_trees = [];
+    my @tokens = ();
     local *spit_parse_tree = sub {
-        push @{$parse_trees}, [@ARG];
+        push @tokens, {
+            'last_pos'  => $ARG[0],
+            'buffer'    => $ARG[1],
+            'rule'      => $ARG[2],
+            'tree'      => $ARG[3],
+        };
     };
-
     $self->_redspans_traverse(\&spit_parse_tree,%colors);
 
-    return $parse_trees;
+    return @tokens;
 }
 
 #---------------------------------------------------------------
@@ -505,9 +511,7 @@ sub _escape_html {
     );
     my $re = join '|',
         map { quotemeta } keys %esc;
-    $str =~ s/($re)      # convert <>"&
-             /$esc{$1}   # to their html escape sequences
-             /gx;
+    $str =~ s/($re)/$esc{$1}/g;
     return $str;
 }
 
@@ -565,8 +569,8 @@ Syntax::Highlight::Perl6 - Perl 6 Syntax Highlighter
     # Prints ANSI escaped color sequences (useful for console and IRC output)
     print $p->ansi_text;
 
-    # Prints the Perl 5 array of parse trees (useful for other libraries)
-    print $p->parse_trees;
+    # Prints an array of token records (useful for other libraries)
+    print $p->tokens;
 
     # Prints VIM-generated syntax highlighted html
     use Text::VimColor;     # This is only needed if you need
@@ -654,34 +658,44 @@ It can inlined if C<inline_resources> option is 1.
 
 Returns a Perl highlighted ANSI escape color string.
 
-=head2 parse_trees
+=head2 tokens
 
-Returns a Perl 5 array containing parse tree records.
-The array consists of one or more of the following record structure:
+Returns an array of hashes containing parsed token records.
+The hash record is structured as:
 
-    +-----------------------------------------------+
-    | Matched  | Matched | Matched   | Parse tree   |
-    | Start    | string  | rule      | separated    |
-    | Position | buffer  | name      | by spaces    |
-    +-----------------------------------------------+
+    +----------------------------------------------+
+    | Matched   | Matched | Matched   | Parse tree |
+    | Last      | string  | rule      | separated  |
+    | Position  | buffer  | name      | by spaces  |
+    |           |         |           |            |
+    | $last_pos | $buffer | $rule     | $tree      |
+    +----------------------------------------------+
 
-An example of the C<parse_trees> method in action:
+An example of the C<tokens> method in action:
 
     use Data::Dumper;
-    print Dumper($p->parse_trees);
+    print Dumper($p->tokens);
 
 The shortened output looks like:
 
-    $VAR1 = [
-          ..
-          [
-            5,             # buffer starts at position 5
-            'foo',         # and is a 'foo'
-            '_scalar',     # and a _scalar rule
-            '...blah blah' # parse tree separated by spaces
-          ]
-          ..
-    ];
+    $VAR1 = {
+      'tree' => '',
+      'rule' => 0,
+      'buffer' => '',
+      'last_pos' => 0
+    };
+    $VAR2 = {
+      'tree' => 'statementlist statement statement_modexpr statement_expr EXPR termish noun value number integer ',
+      'rule' => 'number',
+      'buffer' => '1',
+      'last_pos' => 1
+    };
+    $VAR3 = {
+      'tree' => 'statementlist eat_terminator ',
+      'rule' => 0,
+      'buffer' => ';',
+      'last_pos' => 2
+    };
 
 =head2 vim_html
 
@@ -723,10 +737,6 @@ You have to put "use STD;" before using this module.
     use STD; # this must be first for now
     use Syntax::Highlight::Perl6;
 
-=item * A log file is always created when using this module.
-
-This is a bug and im working on it. Most probably it is STD.pm-related
-
 =item * Directory 'lex' is created when using this module.
 
 STD.pm caches digraphs and the matched rules in there.
@@ -737,11 +747,6 @@ So this is Pretty Voodoo Stuff. Otherwise it will be pretty slow.
 That is related to the creation of the cached 'lex'ing directory by STD.pm.
 I<This happens only once.>
 
-=item * In win32 platforms, I get the following warning: 'cp' is not recognized
-
-This is an STD.pm bug which is currently under investigation. Do not worry about it
-for now.
-
 =back
 
 =head1 AUTHOR
@@ -762,13 +767,13 @@ Thanks guys. I could not have done it without you ;-)
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (C) 2008 by Ahmad Zawawi
+Copyright (C) 2008-2009 by Ahmad Zawawi
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as the Artistic License 2.0
+This program is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
 
 This library also includes the following libraries:
 
-STD.pm by Larry Wall (Artistic License 2.0 - same license)
+STD.pm by Larry Wall (Artistic License 2.0)
 
 JQuery by John Resig (dual licensed under the MIT and GPL licenses).
